;;; DO NOT MODIFY THIS FILE
(if (featurep 'xslt-process-autoloads) (error "Already loaded"))

;;;### (autoloads nil "_pkg" "lisp/_pkg.el")

(package-provide 'xslt-process :version 1.12 :author-version "1.2.1" :type 'regular)

;;;***

;;;### (autoloads (xslt-process-mode) "xslt-process" "lisp/xslt-process.el")

(defcustom xslt-process-additional-classpath nil "*Additional Java classpath to be passed when invoking Bean Shell.\nNote that modifying this won't have any effect until you restart the\nBean Shell. You can do this by killing the *bsh* buffer." :group 'xslt-process :type '(repeat (file :must-match t :tag "Path")))

(defcustom xslt-process-key-binding "" "*Keybinding for invoking the XSLT processor.\nTo enter a normal key, enter its corresponding character. To enter a\nkey with a modifier, either type C-q followed by the desired modified\nkeystroke, e.g. C-q C-c to enter Control c. To enter a function key,\nuse the [f1], [f2] etc. notation." :group 'xslt-process :type '(string :tag "Key"))

(defcustom xslt-process-mode-line-string " XSLT" "*String displayed in the modeline when the xslt-process minor\nmode is active. Set this to nil if you don't want a modeline\nindicator." :group 'xslt-process :type 'string)

(defvar xslt-process-mode nil)

(make-variable-buffer-local 'xslt-process-mode)

(defvar xslt-process-mode-map (make-sparse-keymap))

(autoload 'xslt-process-mode "xslt-process" "\
Minor mode to invoke an XSLT processor on the current buffer.

This mode spawns off a Java Bean Shell process in the background to
run an XSLT processor of your choice. This minor mode makes use of
Emacs-Lisp functionality defined in JDE, the Java Development
Environment for Emacs.

With no argument, this command toggles the xslt-process mode. With a
prefix argument ARG, turn xslt-process minor mode on iff ARG is
positive.

Bindings:
\\[xslt-process-invoke]: Invoke the XSLT processor on the current buffer.

Hooks:
xslt-process-hook is run after the xslt-process minor mode is entered.

For more information please check:

xslt-process:    http://www.geocities.com/SiliconValley/Monitor/7464/
Emacs JDE:       http://sunsite.dk/jde/
Java Bean Shell: http://www.beanshell.org/
" t nil)

(if (fboundp 'add-minor-mode) (add-minor-mode 'xslt-process-mode xslt-process-mode-line-string xslt-process-mode-map nil 'xslt-process-mode) (or (assoc 'xslt-process-mode minor-mode-alist) (setq minor-mode-alist (cons '(xslt-process-mode xslt-process-mode-line-string) minor-mode-alist))) (or (assoc 'xslt-process-mode minor-mode-map-alist) (setq minor-mode-map-alist (cons (cons 'xslt-process-mode xslt-process-mode-map) minor-mode-map-alist))))

;;;***

(provide 'xslt-process-autoloads)
